using System;
using System.IO;
using ICSharpCode.SharpZipLib.Zip;

namespace Bibliotecas.Model
{

    public class Compactador
    {

        private string _arquivoCompactado;
        private string _senha = "";
        private int _nivelCompactacao = 5;
        private ZipOutputStream _zipStream;
        private tipoCompactador _compactador;
        private ZipFile _zipFile;

        public Compactador(tipoCompactador tipo)
        {
            _compactador = tipo;
        }

        public void CriarArquivoCompactado(string novoArquivoCompactado, string senha)
        {

            if (_compactador == tipoCompactador.Compactar)
            {
                _arquivoCompactado = novoArquivoCompactado;
                FileStream fsOut = File.Create(_arquivoCompactado);
                _zipStream = new ZipOutputStream(fsOut);
                _zipStream.SetLevel(_nivelCompactacao);

                if (!string.IsNullOrEmpty(senha))
                    _zipStream.Password = senha;

            }
            else
                Retorno.Erro("o Tipo de compactador no pode ser usado para Criao de Arquivos");
        }

        public void AbrirArquivoCompactado(string ArquivoCompactado)
        {
            if (_compactador == tipoCompactador.Compactar)
            {
                _zipFile = new ZipFile(ArquivoCompactado);
            }
            else
                Retorno.Erro("o Tipo de compactador no pode ser usado para Criao de Arquivos");
        }

        public void DescompactaArquivo(string arquivoCompactado, string pastaDestino, string filtro, string senha)
        {
            if (_compactador == tipoCompactador.Descompactar)
            {

                _arquivoCompactado = arquivoCompactado;

                FastZip fz = new FastZip
                {
                    Password = senha
                };

                fz.ExtractZip(arquivoCompactado, pastaDestino, filtro);
            }
            else
                Retorno.Erro("o Tipo de compactador no pode ser usado para Descompactar Arquivos");
        }

        public int insereArquivos(string[] arquivos)
        {
            if (_compactador == tipoCompactador.Compactar)
            {
                if (_arquivoCompactado != null)
                {
                    foreach (string filename in arquivos)
                    {

                        string entryName = Path.GetFileName(filename);
                        ZipEntry newEntry = new ZipEntry(entryName)
                        {
                            DateTime = DateTime.Now
                        };

                        _zipStream.UseZip64 = UseZip64.Off;
                        _zipStream.PutNextEntry(newEntry);

                        byte[] buffer = new byte[4096];
                        FileStream streamReader = File.OpenRead(filename);

                        int n;
                        do
                        {
                            n = streamReader.Read(buffer, 0, buffer.Length);
                            _zipStream.Write(buffer, 0, n);
                        } while (n > 0);
                        _zipStream.CloseEntry();
                        streamReader.Close();
                    }

                    _zipStream.IsStreamOwner = true;    // Makes the Close also Close the underlying stream
                    _zipStream.Close();

                    return arquivos.Length;
                }
                else
                {
                    Retorno.Erro("Voce precisa criar o arquivo compactado antes de inserir Arquivos.");
                    return 0;
                }
            }
            else
            {
                Retorno.Erro("o Tipo de compactador no pode ser usado para Insero de Arquivos");
                return 0;
            }
        }

        public void insereArquivo(string CaminhoArquivo, string NomeArquivo)
        {
            if (_compactador == tipoCompactador.Compactar)
            {
                if (_arquivoCompactado != null)
                {

                    ZipEntry newEntry = new ZipEntry(NomeArquivo)
                    {
                        DateTime = DateTime.Now
                    };

                    _zipStream.UseZip64 = UseZip64.Off;
                    _zipStream.PutNextEntry(newEntry);

                    byte[] buffer = new byte[4096];
                    FileStream streamReader = File.OpenRead(CaminhoArquivo);

                    int n;
                    do
                    {
                        n = streamReader.Read(buffer, 0, buffer.Length);
                        _zipStream.Write(buffer, 0, n);
                    } while (n > 0);
                    _zipStream.CloseEntry();
                    streamReader.Close();

                    _zipStream.IsStreamOwner = true;    // Makes the Close also Close the underlying stream
                    _zipStream.Close();
                }
                else
                {
                    if (_zipFile != null)
                    {
                        _zipFile.BeginUpdate();
                        _zipFile.Password = "";
                        _zipFile.Add(CaminhoArquivo, NomeArquivo);
                        _zipFile.CommitUpdate();
                        _zipFile.Close();
                    }
                    else
                        Retorno.Erro("Voce precisa criar o arquivo compactado antes de inserir Arquivos.");
                }
            }
            else
            {
                Retorno.Erro("o Tipo de compactador no pode ser usado para Insero de Arquivos");
            }
        }

        public string Senha
        {
            get { return _senha; }
            set { _senha = value; }
        }

        public enum tipoCompactador
        {
            Compactar,
            Descompactar
        }

        public int NivelCompactacao
        {
            get { return _nivelCompactacao; }
            set { _nivelCompactacao = value; }
        }
    }
}
